Object.defineProperty(exports, "__esModule", { value: true });
exports.MainBackend = exports.isAppReady = void 0;
var tslib_1 = require("tslib");
/* eslint-disable max-lines */
var core_1 = require("@sentry/core");
var node_1 = require("@sentry/node");
var types_1 = require("@sentry/types");
var utils_1 = require("@sentry/utils");
var electron_1 = require("electron");
var path_1 = require("path");
var common_1 = require("../common");
var index_1 = require("./index");
var normalize_1 = require("./normalize");
var store_1 = require("./store");
var net_1 = require("./transports/net");
var uploader_1 = require("./uploader");
/** Gets the path to the Sentry cache directory. */
function getCachePath() {
    return path_1.join(electron_1.app.getPath('userData'), 'sentry');
}
/**
 * Retruns a promise that resolves when app is ready.
 */
function isAppReady() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            return [2 /*return*/, (electron_1.app.isReady() ||
                    new Promise(function (resolve) {
                        electron_1.app.once('ready', function () {
                            resolve(true);
                        });
                    }))];
        });
    });
}
exports.isAppReady = isAppReady;
/** Backend implementation for Electron renderer backends. */
var MainBackend = /** @class */ (function (_super) {
    tslib_1.__extends(MainBackend, _super);
    /** Creates a new Electron backend instance. */
    function MainBackend(options) {
        var _this = _super.call(this, options) || this;
        _this._inner = new node_1.NodeBackend(options);
        _this._scopeStore = new store_1.Store(getCachePath(), 'scope_v2', new core_1.Scope());
        var success = true;
        _this._setupScopeListener();
        if (_this._isNativeEnabled()) {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            success = _this._installNativeHandler() && success;
        }
        _this._installIPC();
        return _this;
    }
    /**
     * @inheritDoc
     */
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    MainBackend.prototype.eventFromException = function (exception, hint) {
        return this._inner.eventFromException(exception, hint);
    };
    /**
     * @inheritDoc
     */
    MainBackend.prototype.eventFromMessage = function (message, level, hint) {
        if (level === void 0) { level = types_1.Severity.Info; }
        return this._inner.eventFromMessage(message, level, hint);
    };
    /**
     * @inheritDoc
     */
    MainBackend.prototype.sendEvent = function (event) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        if (event.__INTERNAL_MINIDUMP) {
            electron_1.crashReporter.addExtraParameter('sentry', JSON.stringify(event));
        }
        else {
            this._inner.sendEvent(event);
        }
    };
    /**
     * Uploads the given minidump and attaches event information.
     *
     * @param path A relative or absolute path to the minidump file.
     * @param event Optional event information to add to the minidump request.
     */
    MainBackend.prototype.uploadMinidump = function (path, event) {
        if (event === void 0) { event = {}; }
        if (this._uploader) {
            utils_1.forget(this._uploader.uploadMinidump({ path: path, event: event }));
        }
    };
    /**
     * Setup Transport
     */
    MainBackend.prototype._setupTransport = function () {
        if (!this._options.dsn) {
            // We return the noop transport here in case there is no Dsn.
            return _super.prototype._setupTransport.call(this);
        }
        var transportOptions = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, this._options.transportOptions), (this._options.httpProxy && { httpProxy: this._options.httpProxy })), (this._options.httpsProxy && { httpsProxy: this._options.httpsProxy })), (this._options.caCerts && { caCerts: this._options.caCerts })), { dsn: this._options.dsn });
        if (this._options.transport) {
            return new this._options.transport(transportOptions);
        }
        return new net_1.NetTransport(transportOptions);
    };
    /**
     * Adds a scope listener to persist changes to disk.
     */
    MainBackend.prototype._setupScopeListener = function () {
        var _this = this;
        var hubScope = core_1.getCurrentHub().getScope();
        if (hubScope) {
            hubScope.addScopeListener(function (updatedScope) {
                var cloned = core_1.Scope.clone(updatedScope);
                // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                cloned._eventProcessors = [];
                // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                cloned._scopeListeners = [];
                // if we use the crashpad minidump uploader we have to set extra whenever the scope updates
                if (_this._options.useCrashpadMinidumpUploader === true) {
                    // @ts-ignore __INTERNAL_MINIDUMP is not assignable to event
                    core_1.captureEvent({ __INTERNAL_MINIDUMP: true });
                }
                _this._scopeStore.set(cloned);
            });
        }
    };
    /** Returns whether native reports are enabled. */
    MainBackend.prototype._isNativeEnabled = function () {
        // Mac AppStore builds cannot run the crash reporter due to the sandboxing
        // requirements. In this case, we prevent enabling native crashes entirely.
        // https://electronjs.org/docs/tutorial/mac-app-store-submission-guide#limitations-of-mas-build
        if (process.mas) {
            return false;
        }
        return this._options.enableNative !== false;
    };
    /** Activates the Electron CrashReporter. */
    MainBackend.prototype._installNativeHandler = function () {
        var _this = this;
        // We are only called by the frontend if the SDK is enabled and a valid DSN
        // has been configured. If no DSN is present, this indicates a programming
        // error.
        var dsnString = this._options.dsn;
        if (!dsnString) {
            throw new utils_1.SentryError('Invariant exception: install() must not be called when disabled');
        }
        var dsn = new utils_1.Dsn(dsnString);
        // We will manually submit errors, but CrashReporter requires a submitURL in
        // some versions. Also, provide a productName and companyName, which we will
        // add manually to the event's context during submission.
        electron_1.crashReporter.start({
            companyName: '',
            ignoreSystemCrashHandler: true,
            productName: this._options.appName || common_1.getNameFallback(),
            submitURL: uploader_1.MinidumpUploader.minidumpUrlFromDsn(dsn),
            uploadToServer: this._options.useCrashpadMinidumpUploader || false,
        });
        if (this._options.useSentryMinidumpUploader !== false) {
            // The crashReporter has a method to retrieve the directory
            // it uses to store minidumps in. The structure in this directory depends
            // on the crash library being used (Crashpad or Breakpad).
            var crashesDirectory = electron_1.crashReporter.getCrashesDirectory();
            this._uploader = new uploader_1.MinidumpUploader(dsn, crashesDirectory, getCachePath(), this.getTransport());
            // Flush already cached minidumps from the queue.
            utils_1.forget(this._uploader.flushQueue());
            // Start to submit recent minidump crashes. This will load breadcrumbs and
            // context information that was cached on disk prior to the crash.
            utils_1.forget(this._sendNativeCrashes({}));
        }
        /**
         * Helper function for sending renderer crashes
         */
        var sendRendererCrash = function (contents, details) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var e_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this._sendNativeCrashes(this._getNewEventWithElectronContext(contents, details))];
                    case 1:
                        _a.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        e_1 = _a.sent();
                        // eslint-disable-next-line no-console
                        console.error(e_1);
                        return [3 /*break*/, 3];
                    case 3:
                        core_1.addBreadcrumb({
                            category: 'exception',
                            level: types_1.Severity.Critical,
                            message: 'Renderer Crashed',
                        });
                        return [2 /*return*/];
                }
            });
        }); };
        // Every time a subprocess or renderer crashes, start sending minidumps
        // right away.
        electron_1.app.on('web-contents-created', function (_, contents) {
            var version = utils_1.parseSemver(process.versions.electron);
            var major = version.major || 0;
            var minor = version.minor || 0;
            if ((major === 8 && minor >= 4) || (major === 9 && minor >= 1) || major >= 10) {
                contents.on('render-process-gone', function (_event, details) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                    return tslib_1.__generator(this, function (_a) {
                        switch (_a.label) {
                            case 0: return [4 /*yield*/, sendRendererCrash(contents, details)];
                            case 1:
                                _a.sent();
                                return [2 /*return*/];
                        }
                    });
                }); });
            }
            else {
                contents.on('crashed', function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                    return tslib_1.__generator(this, function (_a) {
                        switch (_a.label) {
                            case 0: return [4 /*yield*/, sendRendererCrash(contents)];
                            case 1:
                                _a.sent();
                                return [2 /*return*/];
                        }
                    });
                }); });
            }
            if (_this._options.enableUnresponsive !== false) {
                contents.on('unresponsive', function () {
                    core_1.captureMessage('BrowserWindow Unresponsive');
                });
            }
        });
        return true;
    };
    /** Installs IPC handlers to receive events and metadata from renderers. */
    MainBackend.prototype._installIPC = function () {
        var _this = this;
        electron_1.ipcMain.on(common_1.IPC_PING, function (event) {
            event.sender.send(common_1.IPC_PING);
        });
        electron_1.ipcMain.on(common_1.IPC_EVENT, function (ipc, jsonEvent) {
            var event;
            try {
                event = JSON.parse(jsonEvent);
            }
            catch (_a) {
                // eslint-disable-next-line no-console
                console.warn('sentry-electron received an invalid IPC_EVENT message');
                return;
            }
            event.contexts = tslib_1.__assign(tslib_1.__assign({}, _this._getNewEventWithElectronContext(ipc.sender).contexts), event.contexts);
            core_1.captureEvent(event);
        });
        electron_1.ipcMain.on(common_1.IPC_SCOPE, function (_, jsonRendererScope) {
            var rendererScope;
            try {
                rendererScope = JSON.parse(jsonRendererScope);
            }
            catch (_a) {
                // eslint-disable-next-line no-console
                console.warn('sentry-electron received an invalid IPC_SCOPE message');
                return;
            }
            var sentScope = core_1.Scope.clone(rendererScope);
            /* eslint-disable @typescript-eslint/no-unsafe-member-access */
            core_1.configureScope(function (scope) {
                if (sentScope._user) {
                    scope.setUser(sentScope._user);
                }
                scope.setTags(sentScope._tags);
                scope.setExtras(sentScope._extra);
                // Since we do not have updates for individual breadcrumbs anymore and only for the whole scope
                // we just add the last added breadcrumb on scope updates
                scope.addBreadcrumb(sentScope._breadcrumbs.pop());
            });
            /* eslint-enable @typescript-eslint/no-unsafe-member-access */
        });
    };
    /** Loads new native crashes from disk and sends them to Sentry. */
    MainBackend.prototype._sendNativeCrashes = function (event) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var uploader, currentCloned, fetchedScope, storedScope, newEvent_1, paths, _oO_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        // Whenever we are called, assume that the crashes we are going to load down
                        // below have occurred recently. This means, we can use the same event data
                        // for all minidumps that we load now. There are two conditions:
                        //
                        //  1. The application crashed and we are just starting up. The stored
                        //     breadcrumbs and context reflect the state during the application
                        //     crash.
                        //
                        //  2. A renderer process crashed recently and we have just been notified
                        //     about it. Just use the breadcrumbs and context information we have
                        //     right now and hope that the delay was not too long.
                        if (this._options.useSentryMinidumpUploader === false) {
                            // In case we are not using the Sentry Minidump uploader we don't want to throw an error
                            return [2 /*return*/];
                        }
                        uploader = this._uploader;
                        if (uploader === undefined) {
                            throw new utils_1.SentryError('Invariant violation: Native crashes not enabled');
                        }
                        currentCloned = core_1.Scope.clone(core_1.getCurrentHub().getScope());
                        fetchedScope = this._scopeStore.get();
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 6, , 7]);
                        storedScope = core_1.Scope.clone(fetchedScope);
                        return [4 /*yield*/, storedScope.applyToEvent(event)];
                    case 2:
                        newEvent_1 = _a.sent();
                        if (!newEvent_1) return [3 /*break*/, 5];
                        return [4 /*yield*/, currentCloned.applyToEvent(newEvent_1)];
                    case 3:
                        newEvent_1 = _a.sent();
                        return [4 /*yield*/, uploader.getNewMinidumps()];
                    case 4:
                        paths = _a.sent();
                        paths.map(function (path) {
                            index_1.captureMinidump(path, tslib_1.__assign({}, newEvent_1));
                        });
                        _a.label = 5;
                    case 5: return [3 /*break*/, 7];
                    case 6:
                        _oO_1 = _a.sent();
                        utils_1.logger.error('Error while sending native crash.');
                        return [3 /*break*/, 7];
                    case 7: return [2 /*return*/];
                }
            });
        });
    };
    /** Returns extra information from a renderer's web contents. */
    MainBackend.prototype._getNewEventWithElectronContext = function (contents, details) {
        var customName = this._options.getRendererName && this._options.getRendererName(contents);
        var electronContext = {
            crashed_process: customName || "renderer[" + contents.id + "]",
            crashed_url: normalize_1.normalizeUrl(contents.getURL()),
        };
        if (details) {
            // We need to do it like this, otherwise we normalize undefined to "[undefined]" in the UI
            electronContext.details = details;
        }
        return {
            contexts: {
                electron: electronContext,
            },
        };
    };
    return MainBackend;
}(core_1.BaseBackend));
exports.MainBackend = MainBackend;
//# sourceMappingURL=backend.js.map